# -*- coding:utf-8 -*-
"""urls"""

from __future__ import unicode_literals

from django.conf import settings

from django.conf.urls import include, url

from coop_cms import sitemap
from coop_cms.settings import (
    get_article_views, install_csrf_failure_view
)
from coop_cms.views.newsletters import NewsletterView, NewsletterPdfView
from coop_cms.views import articles, fragments, homepage, links, navigation, newsletters, medialib, webutils
from coop_cms.views.webutils import DebugErrorCodeView

article_views = get_article_views()
article_view = article_views['article_view']
edit_article_view = article_views['edit_article_view']

install_csrf_failure_view()

urlpatterns = [
    url(r'^htm-editor/', include('coop_html_editor.urls')),

    url(r'^cms/change-template/(?P<article_id>\d*)/$', articles.change_template, name="coop_cms_change_template"),
    url(r'^cms/settings/(?P<article_id>\d*)/$', articles.article_settings, name="coop_cms_article_settings"),
    url(r'^cms/new/$', articles.new_article, name="coop_cms_new_article"),
    url(r'^cms/new/article/$', articles.new_article, name="coop_cms_new_article"),
    url(r'^cms/update-logo/(?P<article_id>\d*)/$', articles.update_logo, name="coop_cms_update_logo"),
    url(r'^cms/articles/$', articles.view_all_articles, name="coop_cms_view_all_articles"),
    url(r'^cms/$', articles.view_all_articles),
    url(r'articles/(?P<slug>[-\w]+)/$', articles.ArticlesByCategoryView.as_view(), name="coop_cms_articles_category"),

    url(r'^cms/fragments/add/$', fragments.add_fragment, name='coop_cms_add_fragment'),
    url(r'^cms/fragments/edit/$', fragments.edit_fragments, name='coop_cms_edit_fragments'),

    url(r'^cms/set-homepage/(?P<article_id>\d*)/$', homepage.set_homepage, name='coop_cms_set_homepage'),

    url(r'^cms/new/link/$', links.new_link, name="coop_cms_new_link"),

    url(r'^cms/tree/(?P<tree_id>\d*)/$', navigation.process_nav_edition, name='navigation_tree'),

    url(r'^cms/newsletter/new/$', newsletters.newsletter_settings, name='coop_cms_new_newsletter'),
    url(
        r'^cms/newsletter/settings/(?P<newsletter_id>\d+)/$',
        newsletters.newsletter_settings,
        name='coop_cms_newsletter_settings'
    ),
    url(
        r'^cms/newsletter/(?P<id>\d+)/$',
        NewsletterView.as_view(),
        name='coop_cms_view_newsletter'
    ),
    url(
        r'^cms/newsletter-pdf/(?P<id>\d+)/$',
        NewsletterPdfView.as_view(),
        name='coop_cms_newsletter_pdf'
    ),
    url(
        r'^cms/newsletter/(?P<id>\d+)/cms_edit/$',
        NewsletterView.as_view(edit_mode=True),
        name='coop_cms_edit_newsletter'),

    url(
        r'^cms/newsletter/change-template/(?P<newsletter_id>\d+)/$',
        newsletters.change_newsletter_template,
        name="coop_cms_change_newsletter_template"
    ),
    url(
        r'^cms/newsletter/test/(?P<newsletter_id>\d+)/$',
        newsletters.test_newsletter,
        name="coop_cms_test_newsletter"
    ),
    url(
        r'^cms/newsletter/schedule/(?P<newsletter_id>\d+)/$',
        newsletters.schedule_newsletter_sending,
        name="coop_cms_schedule_newsletter_sending"
    ),

    url(r'^cms/media-images/$', medialib.show_media, {'media_type': 'image'}, name='coop_cms_media_images'),
    url(r'^cms/media-documents/$', medialib.show_media, {'media_type': 'document'}, name='coop_cms_media_documents'),
    url(
        r'^cms/media-photologue/$',
        medialib.show_media,
        {'media_type': 'photologue'},
        name='coop_cms_media_photologue'
    ),
    url(r'^cms/upload-image/$', medialib.upload_image, name="coop_cms_upload_image"),
    url(r'^cms/upload-doc/$', medialib.upload_doc, name="coop_cms_upload_doc"),
    url(r'^cms/private-download/(?P<doc_id>\d*)/$', medialib.download_doc, name='coop_cms_download_doc'),

    url(r'cms/change-language/$', webutils.change_language, name='coop_cms_change_language'),
    url(r'cms/swicth-language/$', webutils.switch_language_popup, name='coop_cms_switch_language_popup'),

    url(
        r'^cms/hide-accept-cookies-message/',
        webutils.hide_accept_cookies_message,
        name='coop_cms_hide_accept_cookies_message'
    ),
]

if settings.DEBUG:
    urlpatterns += [
        url(
            r'^cms/debug-error-code/(?P<error_code>\d{3})/$',
            DebugErrorCodeView.as_view(),
            name='coop_cms_debug_404'
        ),
    ]

if not getattr(settings, "COOP_CMS_DISABLE_DEFAULT_SITEMAP", False):
    urlpatterns += sitemap.urlpatterns

if 'coop_cms.apps.rss_sync' in settings.INSTALLED_APPS:
    urlpatterns += [
        url(r'^rss-sync/', include('coop_cms.apps.rss_sync.urls')),
    ]

if 'coop_cms.apps.test_app' in settings.INSTALLED_APPS:
    urlpatterns += [
        url(r'^this-should-be-only-in-test-mode', include('coop_cms.apps.test_app.urls')),
    ]

# keep these at the end
urlpatterns += [
    url(r'^(?P<url>[-\w]+)/cms_publish/$', articles.publish_article, name='coop_cms_publish_article'),
    url(r'^(?P<url>[-\w]+)/cms_cancel/$', articles.cancel_edit_article, name='coop_cms_cancel_edit_article'),
    url(r'^$', homepage.homepage, name='coop_cms_homepage'),
    url(r'^(?P<slug>[-\w]+)/cms_edit/$', edit_article_view.as_view(edit_mode=True), name='coop_cms_edit_article'),
    url(r'^(?P<slug>[-\w]+)/$', article_view.as_view(), name='coop_cms_view_article'),
    url(r'^(?P<path>.+)$', articles.AliasView.as_view(), name='coop_cms_view_alias'),
    url(r'^coop_bar/', include('coop_bar.urls')),
]



# -*- coding: utf-8 -*-
"""utils"""

from __future__ import unicode_literals

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.http import HttpResponseRedirect
from django.utils import translation


def activate_lang(lang=None):
    lang = lang or get_language()
    language_codes = [code_and_name[0] for code_and_name in settings.LANGUAGES]
    if not (lang in language_codes):
        # The current language is not defined in settings.LANGUAGE
        # force it to the defined language
        lang = settings.LANGUAGE_CODE[:2]
    translation.activate(lang)


def get_language():
    """returns the language or default language"""
    lang = translation.get_language()
    if lang:
        return lang[:2]
    else:
        return settings.LANGUAGE_CODE[:2]


def get_url_in_language(url, lang_code):
    """returns the url in another language"""
    if lang_code and translation.check_for_language(lang_code):
        path = strip_locale_path(url)[1]
        new_url = make_locale_path(path, lang_code)
        return new_url
    else:
        raise ImproperlyConfigured("{0} is not a valid language".format(lang_code))


def strip_locale_path(locale_path):
    """returns language independent url - /en/home/ --> /home/"""
    elements = locale_path.split('/')
    if len(elements) > 2:
        lang = elements[1]
        if lang in [lang_and_name[0] for lang_and_name in settings.LANGUAGES]:
            del elements[1]
            return lang, '/'.join(elements)
    return '', locale_path


def make_locale_path(path, lang):
    """returns locale url - /home/ --> /en/home/"""
    return '/{0}{1}'.format(lang, path)


def redirect_to_language(url, lang_code):
    """change the language"""
    new_url = get_url_in_language(url, lang_code)
    translation.activate(lang_code)
    return HttpResponseRedirect(new_url)



# -*- coding: utf-8 -*-
"""widgets"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.forms import HiddenInput
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from coop_cms.apps.rss_sync.models import RssSource


def get_button_code(label, url, is_default=False):
    """create a html button"""
    css_class = u' default' if is_default else ''
    html = "&nbsp;&nbsp;"
    html += """<button class="btn btn-primary cust-btn{2}" onclick="window.location=\'{1}\'">{0}</button>""".format(
        label, url, css_class
    )

    #javascript code for moving the button to the submit-row at the bottom of the page
    html += "<script>django.jQuery(function() {django.jQuery('.cust-btn').appendTo('.submit-row')});</script>"
    return html


class AdminCollectRssWidget(HiddenInput):
    """
    Widget for RssSource id
    The id is hidden and replaced by an action button calling the collect_rss_items view
    """

    def render(self, name, value, attrs=None):
        """returns html"""
        widget = super(AdminCollectRssWidget, self).render(name, value, attrs) # pylint: disable=E1002
        html = '{0}'.format(widget)
        html += value
        src_id = RssSource.objects.get(url=value).id
        url = reverse('rss_sync_collect_rss_items', args=[src_id])
        html += get_button_code(_('Collect'), url, True)
        return mark_safe(html)


class AdminCreateArticleWidget(HiddenInput):
    """
    Widget for RssItem id
    The id is hidden and replaced by an action button which is creating a CMS article
    """

    def render(self, name, value, attrs=None):
        """returns html"""
        widget = super(AdminCreateArticleWidget, self).render(name, value, attrs)  # pylint: disable=E1002
        html = '{0}'.format(widget)
        html += '{0}'.format(value)
        url = reverse('rss_sync_create_cms_article', args=[int(value)])
        html += get_button_code(_('Create CMS article'), url)
        return mark_safe(html)



# -*- coding: utf-8 -*-
"""send newsletter"""

from __future__ import unicode_literals, print_function

from datetime import datetime

from django.core.management.base import BaseCommand

from coop_cms.utils import send_newsletter
from coop_cms.models import NewsletterSending


class Command(BaseCommand):
    """send newsletter"""
    help = "send newsletter"

    def add_arguments(self, parser):
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument('email_addresses')

    def handle(self, *args, **options):
        """command"""
        # look for emailing to be sent

        verbose = options.get('verbosity', 1)
        email_addresses = options.get('email_addresses', '')
        
        if email_addresses:
            email_list = email_addresses.split(";")
        else:
            print('usage: python manage.py send_newsletter toto@toto.fr;titi@titi.fr')
        
        sendings = NewsletterSending.objects.filter(scheduling_dt__lte=datetime.now(), sending_dt=None)
        for sending in sendings:
            if verbose:
                print('send_newsletter {1} to {0} addresses'.format(len(email_list), sending.newsletter))
            
            nb_sent = send_newsletter(sending.newsletter, email_list)
            
            if verbose:
                print(nb_sent, "emails sent")
            
            sending.sending_dt = datetime.now()
            sending.save()



# -*- coding: utf-8 -*-
"""Example of CMS app"""

from __future__ import unicode_literals


def get_article_templates(article, user):
    """
    example of functions to get custom templates
    It may depend on article or user
    """
    return (
        ('standard.html', 'Standard'),
        ('homepage.html', 'Homepage'),
        ('blog.html', 'Blog'),
        ('standard_en.html', 'English'),
    )

default_app_config = 'coop_cms.apps.demo_cms.apps.DemoCmsAppConfig'



# -*- coding: utf-8 -*-
"""utils"""

from __future__ import unicode_literals

from re import sub
from sys import stderr
from traceback import print_exc

from django.utils.text import slugify as ascii_slugify
from slugify import slugify as unicode_slugify

from coop_cms.moves import HTMLParser
from coop_cms.settings import get_eastern_languages

from .i18n import get_language


class _DeHTMLParser(HTMLParser):
    """html to text parser"""
    def __init__(self, allow_spaces=False, allow_html_chars=False):
        HTMLParser.__init__(self)
        self._text = []
        self._allow_spaces = allow_spaces
        self._allow_html_chars = allow_html_chars

    def handle_data(self, data):
        """parser"""
        text = data
        if not self._allow_spaces:
            text = sub('[ \t\r\n]+', ' ', text)
        self._text.append(text)

    def handle_entityref(self, name):
        html_char = '&' + name + ";"
        if self._allow_html_chars:
            value = html_char
        else:
            value = self.unescape(html_char).replace('\xa0', ' ')
        self._text.append(value)

    def handle_charref(self, name):
        self.handle_entityref("#" + name)

    def handle_starttag(self, tag, attrs):
        """parser"""
        if tag == 'p':
            self._text.append('\n\n')
        elif tag == 'br':
            self._text.append('\n')

    def handle_startendtag(self, tag, attrs):
        """parser"""
        if tag == 'br':
            self._text.append('\n\n')

    def text(self):
        """parser"""
        return ''.join(self._text).strip()


def dehtml(text, allow_spaces=False, allow_html_chars=False):
    """
    html to text
    copied from http://stackoverflow.com/a/3987802/117092
    """
    try:
        parser = _DeHTMLParser(allow_spaces=allow_spaces, allow_html_chars=allow_html_chars)
        parser.feed(text)
        parser.close()
        return parser.text()
    except Exception:  # pylint: disable=broad-except
        print_exc(file=stderr)
        return text


def slugify(text, lang=None):
    """
    slugify a text. Use different method according to language
    "Here COmme the Sun" --> "here-come-the-sun"
    "Voici l'été" --> "voici-l-ete"
    "Миниаль бом" --> "Миниаль-бом"
    Args:
        text: a text to turn into a slug
        lang: the language of this text
    Returns: a slug
    """
    if lang is None:
        lang = get_language()

    if lang in get_eastern_languages():
        return unicode_slugify(text)
    else:
        return ascii_slugify(text)



# -*- coding: utf-8 -*-
"""
Admin pages for coop_cms
"""

from __future__ import unicode_literals

from django.conf import settings
from django.contrib import admin
from django.http import Http404
from django.utils.translation import ugettext_lazy as _

from coop_cms.forms.articles import ArticleAdminForm
from coop_cms.forms.content import AliasAdminForm
from coop_cms.forms.navigation import NavTypeForm, NavNodeAdminForm
from coop_cms.forms.newsletters import NewsletterItemAdminForm, NewsletterAdminForm
from coop_cms import models
from coop_cms.settings import get_article_class, get_navtree_class, import_module


# The BASE_ADMIN_CLASS can be a Translation admin if needed or regular modelAdmin if not
if 'modeltranslation' in settings.INSTALLED_APPS:
    BASE_ADMIN_CLASS = import_module('modeltranslation.admin').TranslationAdmin
else:
    BASE_ADMIN_CLASS = admin.ModelAdmin


def clear_thumbnails_action(model_admin, request, queryset):
    """This action is used by Image Admin and cause sorl-thumbnails to be reset"""
    for obj in queryset:
        obj.clear_thumbnails()
clear_thumbnails_action.short_description = _("Clear thumbnails")


class NavNodeAdmin(admin.ModelAdmin):
    """Navigation node admin"""
    list_display = ["as_text", "label", 'parent', 'ordering', 'in_navigation', 'content_type', 'object_id']
    list_filter = ['tree', 'in_navigation', 'parent']
    list_editable = ["label", 'parent', 'ordering', 'in_navigation', ]
    ordering = ["tree", "parent", "ordering"]
    form = NavNodeAdminForm


admin.site.register(models.NavNode, NavNodeAdmin)


class NavTypeAdmin(admin.ModelAdmin):
    """Navigation type admin"""
    form = NavTypeForm

admin.site.register(models.NavType, NavTypeAdmin)


class NavTreeAdmin(admin.ModelAdmin):
    """Navigation tree admin"""
    list_display = ['as_text', 'name', 'navtypes_list', 'get_root_nodes_count']
    list_editable = ['name']
    list_filters = ['id']

    def nodes_li(self, tree):
        """display the tree nodes for jstree"""
        root_nodes = tree.get_root_nodes()
        nodes_li = ''.join([node.as_jstree() for node in root_nodes])
        return nodes_li

    def navtypes_list(self, tree):
        """list of navigable types"""
        if tree.types.count() == 0:
            return _('All')
        else:
            return ' - '.join(['{0}'.format(x) for x in tree.types.all()])
    navtypes_list.short_description = _('navigable types')

    def change_view(self, request, object_id, extra_context=None, *args, **kwargs):
        """override the change view"""
        extra_context = extra_context or {}
        try:
            object_id = int(object_id)
        except ValueError:
            #if the object_id is not a valid number, returns 404
            raise Http404
        tree = models.get_navtree_class().objects.get(id=object_id)
        extra_context['navtree'] = tree
        extra_context['navtree_nodes'] = self.nodes_li(tree)
        return super(NavTreeAdmin, self).change_view(
            request, str(object_id), extra_context=extra_context, *args, **kwargs
        )  # pylint: disable=E1002

if get_navtree_class():
    admin.site.register(get_navtree_class(), NavTreeAdmin)


class ArticleAdmin(BASE_ADMIN_CLASS):
    """Article admin"""
    form = ArticleAdminForm
    list_display = [
        'slug', 'title', 'category', 'template_name', 'publication', 'headline', 'in_newsletter', 'modified',
        'login_required', 'is_homepage'
    ]
    list_editable = ['publication', 'headline', 'in_newsletter', 'category']
    readonly_fields = ['created', 'modified', 'is_homepage']
    list_filter = [
        'publication', 'login_required', 'headline', 'in_newsletter', 'sites',
        'category', 'template'
    ]
    date_hierarchy = 'publication_date'
    fieldsets = (
        (_('General'), {'fields': ('slug', 'title', 'subtitle', 'publication', 'login_required', )}),
        (_('Settings'), {
            'fields': ('sites', 'template', 'category', 'headline', 'is_homepage', 'logo', 'in_newsletter', )
        }),
        (_('Advanced'), {'fields': ('publication_date', 'created', 'modified', )}),
        (_('Content'), {'fields': ('content', 'summary', )}),
        (_('Debug'), {'fields': ('temp_logo', )}),
    )
    filter_vertical = ('sites', )

    def get_form(self, request, obj=None, **kwargs):
        """return custom form: It adds the current user"""
        form = super(ArticleAdmin, self).get_form(request, obj, **kwargs)  # pylint: disable=E1002
        form.current_user = request.user
        return form

admin.site.register(get_article_class(), ArticleAdmin)


class MediaFilterFilter(admin.SimpleListFilter):
    """filter by media_filter"""
    title = _('Media filter')

    # Parameter for the filter that will be used in the URL query.
    parameter_name = 'media_filter'

    def lookups(self, request, model_admin):
        """values of the filters"""
        media_filters = models.MediaFilter.objects.all()
        return [(x.id, x.name) for x in media_filters]

    def queryset(self, request, queryset):
        """return values after taken the filter into account"""
        value = self.value()
        if value is None:
            return queryset
        return queryset.filter(filters__id=value)


class ImageAdmin(admin.ModelAdmin):
    """Image admin"""
    list_display = ['admin_image', 'name', 'file', 'size', 'ordering', 'copyright']
    list_filter = [MediaFilterFilter, 'size']
    list_editable = ('ordering', 'copyright')
    search_fields = ['name']
    actions = [clear_thumbnails_action]

admin.site.register(models.Image, ImageAdmin)


class FragmentAdmin(BASE_ADMIN_CLASS):
    """Fragment admin"""
    list_display = ['name', 'position', 'type', 'filter', 'css_class']
    list_filter = ['type', 'filter', 'css_class']

admin.site.register(models.Fragment, FragmentAdmin)


class ArticleCategoryAdmin(admin.ModelAdmin):
    """Article category Admin"""
    list_display = ['name', 'slug', 'ordering', 'in_rss', 'pagination_size']
    list_editable = ['ordering', 'in_rss']
    readonly_fields = ['slug']

admin.site.register(models.ArticleCategory, ArticleCategoryAdmin)


class NewsletterItemAdmin(admin.ModelAdmin):
    """newsletter item Admin"""
    form = NewsletterItemAdminForm
    list_display = ['content_type', 'content_object', 'ordering']
    list_editable = ['ordering']
    fieldsets = (
        (_('Article'), {'fields': ('object_id', 'content_type', 'ordering')}),
    )

admin.site.register(models.NewsletterItem, NewsletterItemAdmin)


class NewsletterAdmin(BASE_ADMIN_CLASS):
    """Newsletter Admin"""

    form = NewsletterAdminForm
    raw_id_fields = ['items']
    list_display = ['subject', 'template', 'source_url']

    def get_form(self, request, obj=None, **kwargs):
        """return custom form: it adds the current user"""
        form = super(NewsletterAdmin, self).get_form(request, obj, **kwargs)  # pylint: disable=E1002
        form.current_user = request.user
        return form

admin.site.register(models.Newsletter, NewsletterAdmin)


class IsDuplicatedFilter(admin.SimpleListFilter):
    """filter items to know if they have a parent"""
    title = _(u'Is duplicated')
    parameter_name = 'duplicated'

    def lookups(self, request, model_admin):
        return [
            (1, _(u'Yes')),
            (2, _(u'No')),
        ]

    def get_duplicate_ids(self, queryset):
        duplicated_ids = []

        for alias in queryset:
            duplicates = models.Alias.objects.filter(path=alias, sites__in=alias.sites.all()).exclude(id=alias.id)
            if duplicates.count() > 0:
                duplicated_ids.append(alias.id)
                duplicated_ids.extend([elt.id for elt in duplicates])
        return duplicated_ids

    def queryset(self, request, queryset):
        value = self.value()
        if value == '1':
            return queryset.filter(id__in=self.get_duplicate_ids(queryset))
        elif value == '2':
            return queryset.exclude(id__in=self.get_duplicate_ids(queryset))
        return queryset


class AliasAdmin(BASE_ADMIN_CLASS):
    """Alias admin"""
    list_display = ['path', 'redirect_url', 'redirect_code']
    list_editable = ['redirect_url', 'redirect_code']
    list_filter = ['redirect_code', IsDuplicatedFilter, 'sites', ]
    search_fields = ['path', 'redirect_url']
    ordering = ['path', ]
    form = AliasAdminForm

admin.site.register(models.Alias, AliasAdmin)

admin.site.register(models.MediaFilter)


class ImageSizeAdmin(admin.ModelAdmin):
    """Image size admin"""
    list_display = ['name', 'size', 'crop']

admin.site.register(models.ImageSize, ImageSizeAdmin)


class LinkAdmin(BASE_ADMIN_CLASS):
    list_display = ('title', 'url', )
    filter_vertical = ('sites', )
    list_filter = ('sites', )
    search_fields = ('title', )

admin.site.register(models.Link, LinkAdmin)

admin.site.register(models.PieceOfHtml)
admin.site.register(models.NewsletterSending)
admin.site.register(models.FragmentType)
admin.site.register(models.FragmentFilter)

admin.site.register(models.Document)


class SiteSettingsAdmin(BASE_ADMIN_CLASS):
    list_display = ('site', 'homepage_url', 'homepage_article', 'sitemap_mode')


admin.site.register(models.SiteSettings, SiteSettingsAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0013_auto_20170607_1508'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='fragment',
            unique_together=set([('type', 'name', 'filter')]),
        ),
    ]



# -*- coding: utf-8 -*-
"""utils"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse, NoReverseMatch


def get_login_url():
    """returns the URL of the login page"""
    try:
        return reverse("auth_login")
    except NoReverseMatch:
        return reverse("login")




# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.test import TestCase
from django.contrib.auth.models import User, Permission
from django.core.urlresolvers import reverse
from coop_cms.settings import get_article_class
from coop_cms.apps.rss_sync.models import RssItem, RssSource
from coop_cms.models import BaseArticle
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from unittest import skipUnless

class BaseTestCase(TestCase):
    def setUp(self):
        self.sync = None
        self.staff = None
        self.editor = None

    def _log_as_synchro(self):
        if not self.sync:
            self.sync = User.objects.create_user('toto', 'toto@toto.fr', 'toto')
            self.sync.is_staff = True
            can_add_items = Permission.objects.get(content_type__app_label='rss_sync', codename='add_rssitem')
            can_change_items = Permission.objects.get(content_type__app_label='rss_sync', codename='change_rssitem')
            self.sync.user_permissions.add(can_add_items)
            self.sync.user_permissions.add(can_change_items)
            self.sync.save()
        
        self.client.login(username='toto', password='toto')
        
    def _log_as_editor(self):
        if not self.editor:
            self.editor = User.objects.create_user('tata', 'tata@tata.fr', 'tata')
            self.editor.is_staff = True
            ct = ContentType.objects.get_for_model(get_article_class())
            can_add_art = Permission.objects.get(content_type__app_label=ct.app_label, codename='add_{0}'.format(ct.model))
            can_change_art = Permission.objects.get(content_type__app_label=ct.app_label, codename='change_{0}'.format(ct.model))
            self.editor.user_permissions.add(can_add_art)
            self.editor.user_permissions.add(can_change_art)
            self.editor.save()
        
        self.client.login(username='tata', password='tata')
        
    def _log_as_staff(self):
        if not self.staff:
            self.staff = User.objects.create_user('titi', 'titi@titi.fr', 'titi')
            self.staff.is_staff = True
            self.staff.save()
        
        self.client.login(username='titi', password='titi')

@skipUnless('coop_cms.apps.rss_sync' in settings.INSTALLED_APPS, "rss_sync not installed installed")
class RssTest(BaseTestCase):

    def _do_test_creatitem_from_source(self, url):
        source = RssSource.objects.create(url=url)
        
        self._log_as_synchro()
        
        url = reverse("rss_sync_collect_rss_items", args=[source.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertTrue(RssItem.objects.filter(source=source).count() > 0)
        for item in RssItem.objects.filter(source=source):
            self.assertTrue(len(item.summary)>0)
            self.assertTrue(len(item.title)>0)
            
    def test_creatitem_from_djangoplanet(self):
        self._do_test_creatitem_from_source("http://www.django-fr.org/planete/rss/")
        
    def test_creatitem_from_blogapidev(self):
        self._do_test_creatitem_from_source("http://www.apidev.fr/blog/rss/")
        
    def test_creatitem_from_crespaca(self):
        #No update_date
        self._do_test_creatitem_from_source("http://www.cresspaca.org/rss_actus.php")
            
    def test_synchro_url_is_not_rss(self):
        source = RssSource.objects.create(url='http://www.apidev.fr/')
        
        self._log_as_synchro()
        
        url = reverse("rss_sync_collect_rss_items", args=[source.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEquals(RssItem.objects.filter(source=source).count(), 0)
            
    def test_synchro_not_an_url(self):
        source = RssSource.objects.create(url='toto')
        self._log_as_synchro()
        
        url = reverse("rss_sync_collect_rss_items", args=[source.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        
        self.assertEquals(RssItem.objects.filter(source=source).count(), 0)
        
    def test_creatitem_permission_required(self):
        source = RssSource.objects.create(url='http://www.apidev.fr/blog/rss/')
        
        url = reverse("rss_sync_collect_rss_items", args=[source.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 403)
        self.assertTrue(RssItem.objects.filter(source=source).count() == 0)
        
    def test_creatitem_staff_is_not_enough(self):
        source = RssSource.objects.create(url='http://www.apidev.fr/blog/rss/')
        self._log_as_staff()
        url = reverse("rss_sync_collect_rss_items", args=[source.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 403)
        self.assertTrue(RssItem.objects.filter(source=source).count() == 0)
        
    def test_creatitem_no_source(self):
        self._log_as_synchro()
        url = reverse("rss_sync_collect_rss_items", args=[1])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 404)
        self.assertTrue(RssItem.objects.count() == 0)
                             

@skipUnless('coop_cms.apps.rss_sync' in settings.INSTALLED_APPS, "rss_sync not installed installed")
class CreateArticleTest(BaseTestCase):
    def test_create_article_from_rssitem(self):
        source = RssSource.objects.create(url='http://www.apidev.fr/blog/rss/')
        item = RssItem.objects.create(title="hey-hey", summary="there is nothing i can say", source=source)
        
        self._log_as_editor()
        
        url = reverse("rss_sync_create_cms_article", args=[item.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 200)
        
        art = get_article_class().objects.get(title=item.title)
        self.assertEqual(item.summary, art.content)
        self.assertEqual(art.publication, BaseArticle.DRAFT)
        
    def test_create_article_from_rssitem_permission_required(self):
        source = RssSource.objects.create(url='http://www.apidev.fr/blog/rss/')
        item = RssItem.objects.create(title="heyhey", summary="nothing i can say", source=source)
        
        url = reverse("rss_sync_create_cms_article", args=[item.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 403)
        
        self.assertEqual(0, get_article_class().objects.count())
        
    def test_create_article_from_rssitem_staff_is_not_enough(self):
        source = RssSource.objects.create(url='http://www.apidev.fr/blog/rss/')
        item = RssItem.objects.create(title="heyhey", summary="nothing i can say", source=source)
        
        self._log_as_staff()
        
        url = reverse("rss_sync_create_cms_article", args=[item.id])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 403)
        
        self.assertEqual(0, get_article_class().objects.count())
        
    def test_creatie_article_item_not_found(self):
        self._log_as_editor()
        url = reverse("rss_sync_create_cms_article", args=[1])
        response = self.client.get(url, follow=True)
        self.assertEqual(response.status_code, 404)
        self.assertTrue(get_article_class().objects.count() == 0)



# -*- coding: utf-8 -*-
"""
coop_cms manage compatibilty with django and python versions
"""

import json
import sys

from django import VERSION
from django.conf import settings
from django.template import RequestContext, Context


if sys.version_info[0] < 3:
    # Python 2
    from StringIO import StringIO
    from StringIO import StringIO as BytesIO
    from HTMLParser import HTMLParser
else:
    # Python 3
    from io import StringIO as StringIO
    from io import BytesIO as BytesIO
    from html.parser import HTMLParser as BaseHTMLParser

    class HTMLParser(BaseHTMLParser):
        def __init__(self):
            BaseHTMLParser.__init__(self, convert_charrefs=False)


try:
    from django.utils.deprecation import MiddlewareMixin
except ImportError:
    MiddlewareMixin = object


if VERSION >= (1, 9, 0):
    from wsgiref.util import FileWrapper
else:
    from django.core.servers.basehttp import FileWrapper


if VERSION >= (1, 8, 0):
    from unittest import SkipTest
else:
    # Deprecated in Django 1.9
    from django.utils.unittest import SkipTest


def make_context(request, context_dict, force_dict=True):
    """"""
    if VERSION >= (1, 9, 0):
        if force_dict:
            context = dict(context_dict)
            if request:
                context['request'] = request
                context['MEDIA_URL'] = settings.MEDIA_URL
                context['user'] = request.user
        else:
            if request:
                context = RequestContext(request, context_dict)
            else:
                context = Context(context_dict)
    else:
        if request:
            context = RequestContext(request, context_dict)
        else:
            context = Context(context_dict)
    return context


def get_response_json(response):
    if sys.version_info[0] < 3:
        return json.loads(response.content)
    else:
        return response.json()



# -*- coding: utf-8 -*-
"""
Demo
"""

from __future__ import unicode_literals

from django.apps import AppConfig

class DemoCmsAppConfig(AppConfig):
    name = 'coop_cms.apps.demo_cms'
    verbose_name = "coop CMS > Demo CMS"



# -*- coding: utf-8 -*-
"""
Download articles from RSS feeds
"""

from __future__ import unicode_literals


from django.apps import AppConfig

class RssSyncAppConfig(AppConfig):
    name = 'coop_cms.apps.rss_sync'
    verbose_name = "RSS Synchronization"



# -*- coding: utf-8 -*-
"""
Example of custom forms
"""

from __future__ import unicode_literals

import floppyforms.__future__ as forms

from coop_cms.forms.articles import ArticleForm as CmsArticleForm
from coop_cms.forms.newsletters import NewsletterForm
from coop_cms.settings import get_article_class
from coop_cms.models import ArticleCategory


class ArticleForm(CmsArticleForm):
    """Custom article form"""

    class Meta(CmsArticleForm.Meta):
        model = get_article_class()
        fields = CmsArticleForm.Meta.fields + ('author',)


class SortableNewsletterForm(NewsletterForm):
    """Example of newsletter form"""
    # cleaned_data = None

    sortable = forms.CharField(required=False, widget=forms.HiddenInput())

    class Media(NewsletterForm.Media):
        js = NewsletterForm.Media.js + ('js/jquery.sortElements.js',)

    def save(self, *args, **kwargs):
        """override save"""
        ret = super(SortableNewsletterForm, self).save(*args, **kwargs)

        order = self.cleaned_data['sortable']
        if order:
            order = [int(x) for x in order.split(';')]
            for art_id in order:
                section = ArticleCategory.objects.get(id=art_id)
                section.ordering = order.index(art_id) + 1
                section.save()

        return ret



# -*- coding: utf-8 -*-
"""Boptstrap-css friendly forms"""

from __future__ import unicode_literals

import types

import floppyforms.__future__ as forms

from coop_cms.templatetags.coop_utils import is_checkbox


def _build_attrs(widget, base_attrs, extra_attrs=None, **kwargs):
    """Patch > Helper function for building an attribute dictionary."""
    attrs = base_attrs.copy()
    if widget.attrs:
        attrs.update(widget.attrs)
    if 'class' in attrs:
        val = attrs['class']
        attrs['class'] = val + " form-control"
    else:
        attrs['class'] = "form-control"
    if extra_attrs is not None:
        attrs.update(extra_attrs)
    if kwargs:
        attrs.update(kwargs)
    return attrs


class BootstrapableMixin(object):

    def _bs_patch_field_class(self):
        """Patch fields for adding form-control class to widget"""
        for field_name in self.fields:
            field = self.fields[field_name]
            self._bs_patch_field(field)

    @staticmethod
    def _bs_patch_field(field):
        if not is_checkbox(field):
            # replace the `build_attrs` method of the widget with a custom version whic adds the
            # form-control class to attrs
            field.widget.build_attrs = types.MethodType(_build_attrs, field.widget)


class Form(forms.Form, BootstrapableMixin):

    def __init__(self, *args, **kwargs):
        super(Form, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()


class ModelForm(forms.ModelForm, BootstrapableMixin):

    def __init__(self, *args, **kwargs):
        super(ModelForm, self).__init__(*args, **kwargs)
        self._bs_patch_field_class()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""package the lib"""

from __future__ import unicode_literals

try:
    from setuptools import setup, find_packages
except ImportError:
    import ez_setup
    ez_setup.use_setuptools()
    from setuptools import setup, find_packages

VERSION = __import__('coop_cms').__version__

import os


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name='apidev-coop_cms',
    version=VERSION,
    description='Small CMS built around a tree navigation open to any django models',
    packages=find_packages(),
    include_package_data=True,
    author='Luc Jean',
    author_email='ljean@apidev.fr',
    license='BSD',
    zip_safe=False,
    install_requires=[
        'django >= 1.11, <2.0',
        'django-extensions',
        'sorl-thumbnail',
        'apidev-coop_colorbox >= 1.2.7',
        'apidev-coop_bar >= 1.3.2',
        'coop_html_editor >= 1.2.1',
        'feedparser',
        'beautifulsoup4',
        'django-filetransfers',
        'model_mommy',
        'unicode-slugify',
    ],
    long_description=open('README.rst').read(),
    url='https://github.com/ljean/coop_cms/',
    download_url='https://github.com/ljean/coop_cms/tarball/master',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Natural Language :: English',
        'Natural Language :: French',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
    ],
)



# -*- coding: utf-8 -*-
"""Rss feed"""

from __future__ import unicode_literals

from django.contrib.syndication.views import Feed

from coop_cms.settings import get_article_class
from coop_cms.utils import dehtml


class ArticleFeed(Feed):
    """Rss feed for article"""
    title = ""
    link = ""
    description = ""

    def items(self):
        """list of items"""
        article_class = get_article_class()
        return article_class.objects.filter(category__in_rss=True).order_by('-publication_date')

    def item_title(self, item):
        """title of an item"""
        return dehtml(item.title)

    def item_description(self, item):
        """description of an item"""
        return item.summary



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0006_image_copyright'),
    ]

    operations = [
        migrations.AddField(
            model_name='newsletter',
            name='newsletter_date',
            field=models.DateField(default=None, null=True, verbose_name='newsletter date', blank=True),
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import forms
from django.utils.translation import ugettext_lazy as _

from registration.forms import RegistrationFormUniqueEmail

from coop_cms.bs_forms import BootstrapableMixin


class RegistrationFormUniqueEmailAndTermsOfService(BootstrapableMixin, RegistrationFormUniqueEmail):
    
    terms_of_service = forms.BooleanField(
        widget=forms.CheckboxInput,
        label=_('I have read and agree to the Terms of Service'),
        error_messages={'required': _("You must agree to the terms to register")}
    )

    def __init__(self, *args, **kwargs):
        super(RegistrationFormUniqueEmailAndTermsOfService, self).__init__(*args, **kwargs)
        self.fields['username'].required = False
        self.fields['username'].widget = forms.HiddenInput()
        self._bs_patch_field_class()

    def clean(self):
        ret = super(RegistrationFormUniqueEmailAndTermsOfService, self).clean()
        email = self.cleaned_data.get('email', '')
        if email:
            self.cleaned_data['username'] = email[:30]
        return ret



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TestClass',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('field1', models.TextField()),
                ('field2', models.TextField()),
                ('field3', models.CharField(max_length=100, blank=True)),
                ('bool_field', models.BooleanField(default=False)),
                ('int_field', models.IntegerField(default=0)),
                ('float_field', models.FloatField(default=0.0)),
                ('other_field', models.CharField(max_length=100)),
            ],
        ),
        migrations.CreateModel(
            name='TestTag',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100)),
            ],
        ),
        migrations.AddField(
            model_name='testclass',
            name='tags',
            field=models.ManyToManyField(to='test_app.TestTag', blank=True),
        ),
    ]
